@@ -197,8 +197,7 @@ static gboolean decode_reg_status(struct netreg_data *nd, const guint8 *msg,
 {
 	GIsiSubBlockIter iter;
 
-	if (!g_isi_sb_iter_init(msg, len, &iter, false))
-		return FALSE;
+	g_isi_sb_iter_init(&iter, msg, len, 0);
 
 	while (g_isi_sb_iter_is_valid(&iter)) {
 
@@ -396,8 +395,7 @@ static bool name_get_resp_cb(GIsiClient *client, const void *restrict data,
 		goto error;
 	}
 
-	if (!g_isi_sb_iter_init(msg+7, len-7, &iter, false))
-		goto error;
+	g_isi_sb_iter_init(&iter, msg, len, 7);
 
 	while (g_isi_sb_iter_is_valid(&iter)) {
 
@@ -510,8 +508,7 @@ static bool available_resp_cb(GIsiClient *client, const void *restrict data,
 	total = msg[2] / 2;
 	list = alloca(total * sizeof(struct ofono_network_operator));
 
-	if (!g_isi_sb_iter_init(msg + 3, len - 3, &iter, false))
-		goto error;
+	g_isi_sb_iter_init(&iter, msg, len, 3);
 
 	while (g_isi_sb_iter_is_valid(&iter)) {
 
@@ -708,14 +705,14 @@ out:
 }
 
 static void isi_register_manual(struct ofono_netreg *netreg,
-				const struct ofono_network_operator *oper,
+				const char *mcc, const char *mnc,
 				ofono_netreg_register_cb_t cb, void *data)
 {
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct isi_cb_data *cbd = isi_cb_data_new(netreg, cb, data);
 
 	guint8 buffer[3] = { 0 };
-	guint8 *bcd = mccmnc_to_bcd(oper->mcc, oper->mnc, buffer);
+	guint8 *bcd = mccmnc_to_bcd(mcc, mnc, buffer);
 
 	const unsigned char msg[] = {
 		NET_SET_REQ,
@@ -767,8 +764,7 @@ static void rat_ind_cb(GIsiClient *client, const void *restrict data,
 	if (!msg || len < 3 || msg[0] != NET_RAT_IND)
 		return;
 
-	if (!g_isi_sb_iter_init(msg + 3, len - 3, &iter, false))
-		return;
+	g_isi_sb_iter_init(&iter, msg, len, 3);
 
 	while (g_isi_sb_iter_is_valid(&iter)) {
 
@@ -822,8 +818,7 @@ static bool rat_resp_cb(GIsiClient *client, const void *restrict data,
 		return true;
 	}
 
-	if (!g_isi_sb_iter_init(msg + 3, len - 3, &iter, false))
-		return true;
+	g_isi_sb_iter_init(&iter, msg, len, 3);
 
 	while (g_isi_sb_iter_is_valid(&iter)) {
 
@@ -891,8 +886,7 @@ static bool rssi_resp_cb(GIsiClient *client, const void *restrict data,
 		goto error;
 	}
 
-	if (!g_isi_sb_iter_init(msg + 3, len - 3, &iter, false))
-		goto error;
+	g_isi_sb_iter_init(&iter, msg, len, 3);
 
 	while (g_isi_sb_iter_is_valid(&iter)) {
 
@@ -984,7 +978,8 @@ static gboolean isi_netreg_register(gpointer user)
 	return FALSE;
 }
 
-static void reachable_cb(GIsiClient *client, bool alive, void *opaque)
+static void reachable_cb(GIsiClient *client, bool alive, uint16_t object,
+				void *opaque)
 {
 	struct ofono_netreg *netreg = opaque;
 
