@@ -36,9 +36,13 @@
 #include "util.h"
 #include "smsutil.h"
 #include "simutil.h"
+#include "storage.h"
 
 #define CBS_MANAGER_INTERFACE "org.ofono.CbsManager"
 
+#define SETTINGS_STORE "cbs"
+#define SETTINGS_GROUP "Settings"
+
 static GSList *g_drivers = NULL;
 
 enum etws_topic_type {
@@ -69,6 +73,9 @@ struct ofono_cbs {
 	int ci;
 	char mnc[OFONO_MAX_MNC_LENGTH + 1];
 	char mcc[OFONO_MAX_MCC_LENGTH + 1];
+	ofono_bool_t powered;
+	GKeyFile *settings;
+	char *imsi;
 	const struct ofono_cbs_driver *driver;
 	void *driver_data;
 	struct ofono_atom *atom;
@@ -179,11 +186,19 @@ void ofono_cbs_notify(struct ofono_cbs *cbs, const unsigned char *pdu,
 	if (cbs->assembly == NULL)
 		return;
 
+	if (!cbs->powered)
+		return;
+
 	if (!cbs_decode(pdu, pdu_len, &c)) {
 		ofono_error("Unable to decode CBS PDU");
 		return;
 	}
 
+	if (cbs_topic_in_range(c.message_identifier, cbs->efcbmid_contents)) {
+		__ofono_cbs_sim_download(cbs->sim, pdu, pdu_len);
+		return;
+	}
+
 	if (!cbs_dcs_decode(c.dcs, &udhi, &cls, &charset, &comp, NULL, NULL)) {
 		ofono_error("Unknown / Reserved DCS.  Ignoring");
 		return;
@@ -267,6 +282,9 @@ static DBusMessage *cbs_get_properties(DBusConnection *conn,
 					OFONO_PROPERTIES_ARRAY_SIGNATURE,
 					&dict);
 
+	ofono_dbus_dict_append(&dict, "Powered", DBUS_TYPE_BOOLEAN,
+				&cbs->powered);
+
 	topics = cbs_topic_ranges_to_string(cbs->topics);
 	ofono_dbus_dict_append(&dict, "Topics", DBUS_TYPE_STRING, &topics);
 	g_free(topics);
@@ -276,6 +294,28 @@ static DBusMessage *cbs_get_properties(DBusConnection *conn,
 	return reply;
 }
 
+static char *cbs_topics_to_str(struct ofono_cbs *cbs, GSList *user_topics)
+{
+	GSList *topics = NULL;
+	char *topic_str;
+	struct cbs_topic_range etws_range = { 4352, 4356 };
+
+	topics = g_slist_append(topics, &etws_range);
+
+	if (user_topics != NULL)
+		topics = g_slist_concat(topics,
+					g_slist_copy(user_topics));
+
+	if (cbs->efcbmid_contents != NULL)
+		topics = g_slist_concat(topics,
+					g_slist_copy(cbs->efcbmid_contents));
+
+	topic_str = cbs_topic_ranges_to_string(topics);
+	g_slist_free(topics);
+
+	return topic_str;
+}
+
 static void cbs_set_topics_cb(const struct ofono_error *error, void *data)
 {
 	struct ofono_cbs *cbs = data;
@@ -315,9 +355,8 @@ static DBusMessage *cbs_set_topics(struct ofono_cbs *cbs, const char *value,
 					DBusMessage *msg)
 {
 	GSList *topics;
-	GSList *etws_topics = NULL;
 	char *topic_str;
-	struct cbs_topic_range etws_range = { 4352, 4356 };
+	struct ofono_error error;
 
 	topics = cbs_extract_topic_ranges(value);
 
@@ -329,20 +368,108 @@ static DBusMessage *cbs_set_topics(struct ofono_cbs *cbs, const char *value,
 
 	cbs->new_topics = topics;
 
-	if (topics != NULL)
-		etws_topics = g_slist_copy(topics);
-
-	etws_topics = g_slist_append(etws_topics, &etws_range);
-	topic_str = cbs_topic_ranges_to_string(etws_topics);
-	g_slist_free(etws_topics);
-
 	cbs->pending = dbus_message_ref(msg);
+
+	if (!cbs->powered) {
+		error.type = OFONO_ERROR_TYPE_NO_ERROR;
+		cbs_set_topics_cb(&error, cbs);
+		return NULL;
+	}
+
+	topic_str = cbs_topics_to_str(cbs, topics);
 	cbs->driver->set_topics(cbs, topic_str, cbs_set_topics_cb, cbs);
 	g_free(topic_str);
 
 	return NULL;
 }
 
+static void cbs_set_powered_cb(const struct ofono_error *error, void *data)
+{
+	struct ofono_cbs *cbs = data;
+	const char *path = __ofono_atom_get_path(cbs->atom);
+	DBusConnection *conn = ofono_dbus_get_connection();
+	DBusMessage *reply;
+
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
+		ofono_debug("Setting Cell Broadcast topics failed");
+		if (!cbs->pending)
+			return;
+
+		__ofono_dbus_pending_reply(&cbs->pending,
+					__ofono_error_failed(cbs->pending));
+		return;
+	}
+
+	cbs->powered = !cbs->powered;
+
+	if (cbs->settings) {
+		g_key_file_set_boolean(cbs->settings, SETTINGS_GROUP,
+					"Powered", cbs->powered);
+		storage_sync(cbs->imsi, SETTINGS_STORE, cbs->settings);
+	}
+
+	ofono_dbus_signal_property_changed(conn, path,
+						CBS_MANAGER_INTERFACE,
+						"Powered",
+						DBUS_TYPE_BOOLEAN,
+						&cbs->powered);
+
+	if (!cbs->pending)
+		return;
+
+	reply = dbus_message_new_method_return(cbs->pending);
+	__ofono_dbus_pending_reply(&cbs->pending, reply);
+}
+
+static DBusMessage *cbs_set_powered(struct ofono_cbs *cbs, gboolean value,
+					DBusMessage *msg)
+{
+	const char *path = __ofono_atom_get_path(cbs->atom);
+	DBusConnection *conn = ofono_dbus_get_connection();
+	char *topic_str;
+
+	if (cbs->powered == value)
+		goto reply;
+
+	if (!cbs->driver->set_topics)
+		goto done;
+
+	if (msg)
+		cbs->pending = dbus_message_ref(msg);
+
+	if (value) {
+		topic_str = cbs_topics_to_str(cbs, cbs->topics);
+		cbs->driver->set_topics(cbs, topic_str,
+					cbs_set_powered_cb, cbs);
+		g_free(topic_str);
+	} else {
+		cbs->driver->clear_topics(cbs, cbs_set_powered_cb, cbs);
+	}
+
+	return NULL;
+
+done:
+	cbs->powered = value;
+
+	if (cbs->settings) {
+		g_key_file_set_boolean(cbs->settings, SETTINGS_GROUP,
+					"Powered", cbs->powered);
+		storage_sync(cbs->imsi, SETTINGS_STORE, cbs->settings);
+	}
+
+	ofono_dbus_signal_property_changed(conn, path,
+						CBS_MANAGER_INTERFACE,
+						"Powered",
+						DBUS_TYPE_BOOLEAN,
+						&cbs->powered);
+
+reply:
+	if (msg)
+		return dbus_message_new_method_return(msg);
+
+	return NULL;
+}
+
 static DBusMessage *cbs_set_property(DBusConnection *conn, DBusMessage *msg,
 					void *data)
 {
@@ -368,6 +495,17 @@ static DBusMessage *cbs_set_property(DBusConnection *conn, DBusMessage *msg,
 
 	dbus_message_iter_recurse(&iter, &var);
 
+	if (!strcmp(property, "Powered")) {
+		dbus_bool_t value;
+
+		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_BOOLEAN)
+			return __ofono_error_invalid_args(msg);
+
+		dbus_message_iter_get_basic(&var, &value);
+
+		return cbs_set_powered(cbs, value, msg);
+	}
+
 	if (!strcmp(property, "Topics")) {
 		const char *value;
 
@@ -437,6 +575,13 @@ static void cbs_unregister(struct ofono_atom *atom)
 		cbs->new_topics = NULL;
 	}
 
+	if (cbs->efcbmid_length) {
+		cbs->efcbmid_length = 0;
+		g_slist_foreach(cbs->efcbmid_contents, (GFunc)g_free, NULL);
+		g_slist_free(cbs->efcbmid_contents);
+		cbs->efcbmid_contents = NULL;
+	}
+
 	cbs->sim = NULL;
 
 	if (cbs->reset_source) {
@@ -447,6 +592,16 @@ static void cbs_unregister(struct ofono_atom *atom)
 			__ofono_netreg_set_base_station_name(cbs->netreg, NULL);
 	}
 
+	cbs->powered = FALSE;
+
+	if (cbs->settings) {
+		storage_close(cbs->imsi, SETTINGS_STORE, cbs->settings, TRUE);
+
+		g_free(cbs->imsi);
+		cbs->imsi = NULL;
+		cbs->settings = NULL;
+	}
+
 	if (cbs->netreg_watch) {
 		if (cbs->location_watch) {
 			__ofono_netreg_remove_status_watch(cbs->netreg,
@@ -514,9 +669,7 @@ struct ofono_cbs *ofono_cbs_create(struct ofono_modem *modem,
 	return cbs;
 }
 
-static void sim_cbmi_read_cb(int ok,
-				enum ofono_sim_file_structure structure,
-				int length, int record,
+static void sim_cbmi_read_cb(int ok, int length, int record,
 				const unsigned char *data,
 				int record_length, void *userdata)
 {
@@ -528,9 +681,6 @@ static void sim_cbmi_read_cb(int ok,
 	if (!ok)
 		return;
 
-	if (structure != OFONO_SIM_FILE_STRUCTURE_TRANSPARENT)
-		return;
-
 	if ((length % 2) == 1 || length < 2)
 		return;
 
@@ -570,9 +720,7 @@ static void sim_cbmi_read_cb(int ok,
 	cbs->efcbmi_contents = NULL;
 }
 
-static void sim_cbmir_read_cb(int ok,
-				enum ofono_sim_file_structure structure,
-				int length, int record,
+static void sim_cbmir_read_cb(int ok, int length, int record,
 				const unsigned char *data,
 				int record_length, void *userdata)
 {
@@ -585,9 +733,6 @@ static void sim_cbmir_read_cb(int ok,
 	if (!ok)
 		return;
 
-	if (structure != OFONO_SIM_FILE_STRUCTURE_TRANSPARENT)
-		return;
-
 	if ((length % 4) != 0)
 		return;
 
@@ -629,9 +774,7 @@ static void sim_cbmir_read_cb(int ok,
 	cbs->efcbmir_contents = NULL;
 }
 
-static void sim_cbmid_read_cb(int ok,
-				enum ofono_sim_file_structure structure,
-				int length, int record,
+static void sim_cbmid_read_cb(int ok, int length, int record,
 				const unsigned char *data,
 				int record_length, void *userdata)
 {
@@ -639,13 +782,11 @@ static void sim_cbmid_read_cb(int ok,
 	unsigned short mi;
 	int i;
 	char *str;
+	GSList *contents = NULL;
 
 	if (!ok)
 		return;
 
-	if (structure != OFONO_SIM_FILE_STRUCTURE_TRANSPARENT)
-		return;
-
 	if ((length % 2) == 1 || length < 2)
 		return;
 
@@ -663,37 +804,45 @@ static void sim_cbmid_read_cb(int ok,
 		range->min = mi;
 		range->max = mi;
 
-		cbs->efcbmid_contents = g_slist_prepend(cbs->efcbmid_contents,
-							range);
+		contents = g_slist_prepend(contents, range);
 	}
 
-	if (cbs->efcbmid_contents == NULL)
+	if (contents == NULL)
 		return;
 
-	cbs->efcbmid_contents = g_slist_reverse(cbs->efcbmid_contents);
+	cbs->efcbmid_contents = g_slist_reverse(contents);
 
 	str = cbs_topic_ranges_to_string(cbs->efcbmid_contents);
 	ofono_debug("Got cbmid: %s", str);
 	g_free(str);
-
-	cbs->efcbmid_length = 0;
-	g_slist_foreach(cbs->efcbmid_contents, (GFunc)g_free, NULL);
-	g_slist_free(cbs->efcbmid_contents);
-	cbs->efcbmid_contents = NULL;
 }
 
 static void cbs_got_imsi(struct ofono_cbs *cbs)
 {
 	const char *imsi = ofono_sim_get_imsi(cbs->sim);
+	gboolean powered;
 
 	ofono_debug("Got IMSI: %s", imsi);
 
+	cbs->settings = storage_open(imsi, SETTINGS_STORE);
+	if (cbs->settings == NULL)
+		return;
+
+	cbs->imsi = g_strdup(imsi);
+
+	powered = g_key_file_get_boolean(cbs->settings, SETTINGS_GROUP,
+						"Powered", NULL);
+	cbs_set_powered(cbs, powered, NULL);
+
 	ofono_sim_read(cbs->sim, SIM_EFCBMI_FILEID,
-				sim_cbmi_read_cb, cbs);
+			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+			sim_cbmi_read_cb, cbs);
 	ofono_sim_read(cbs->sim, SIM_EFCBMIR_FILEID,
-				sim_cbmir_read_cb, cbs);
+			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+			sim_cbmir_read_cb, cbs);
 	ofono_sim_read(cbs->sim, SIM_EFCBMID_FILEID,
-				sim_cbmid_read_cb, cbs);
+			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+			sim_cbmid_read_cb, cbs);
 }
 
 static gboolean reset_base_station_name(gpointer user)
@@ -712,7 +861,7 @@ out:
 }
 
 static void cbs_location_changed(int status, int lac, int ci, int tech,
-					const struct ofono_network_operator *op,
+					const char *mcc, const char *mnc,
 					void *data)
 {
 	struct ofono_cbs *cbs = data;
@@ -720,9 +869,9 @@ static void cbs_location_changed(int status, int lac, int ci, int tech,
 	gboolean lac_changed = FALSE;
 	gboolean ci_changed = FALSE;
 
-	DBG("%d, %d, %d, %d, %p", status, lac, ci, tech, op);
+	DBG("%d, %d, %d, %d, %s%s", status, lac, ci, tech, mcc, mnc);
 
-	if (op == NULL) {
+	if (!mcc || !mnc) {
 		if (cbs->mcc[0] == '\0' && cbs->mnc[0] == '\0')
 			return;
 
@@ -733,9 +882,9 @@ static void cbs_location_changed(int status, int lac, int ci, int tech,
 		goto out;
 	}
 
-	if (strcmp(cbs->mcc, op->mcc) || strcmp(cbs->mnc, op->mnc)) {
-		memcpy(cbs->mcc, op->mcc, sizeof(cbs->mcc));
-		memcpy(cbs->mnc, op->mnc, sizeof(cbs->mnc));
+	if (strcmp(cbs->mcc, mcc) || strcmp(cbs->mnc, mnc)) {
+		memcpy(cbs->mcc, mcc, sizeof(cbs->mcc));
+		memcpy(cbs->mnc, mnc, sizeof(cbs->mnc));
 
 		plmn_changed = TRUE;
 		goto out;
@@ -779,10 +928,12 @@ static void netreg_watch(struct ofono_atom *atom,
 				void *data)
 {
 	struct ofono_cbs *cbs = data;
-	const struct ofono_network_operator *op;
+	const char *mcc;
+	const char *mnc;
 
 	if (cond == OFONO_ATOM_WATCH_CONDITION_UNREGISTERED) {
 		cbs->location_watch = 0;
+		cbs->netreg = 0;
 		return;
 	}
 
@@ -790,11 +941,12 @@ static void netreg_watch(struct ofono_atom *atom,
 	cbs->location_watch = __ofono_netreg_add_status_watch(cbs->netreg,
 					cbs_location_changed, cbs, NULL);
 
-	op = ofono_netreg_get_operator(cbs->netreg);
+	mcc = ofono_netreg_get_mcc(cbs->netreg);
+	mnc = ofono_netreg_get_mnc(cbs->netreg);
 
-	if (op) {
-		memcpy(cbs->mcc, op->mcc, sizeof(cbs->mcc));
-		memcpy(cbs->mnc, op->mnc, sizeof(cbs->mnc));
+	if (mcc && mnc) {
+		memcpy(cbs->mcc, mcc, sizeof(cbs->mcc));
+		memcpy(cbs->mnc, mnc, sizeof(cbs->mnc));
 	} else {
 		memset(cbs->mcc, 0, sizeof(cbs->mcc));
 		memset(cbs->mnc, 0, sizeof(cbs->mnc));
