@@ -71,8 +71,9 @@ struct ofono_gprs {
 	int technology;
 	int flags;
 	int next_context_id;
-	int cid_min;
-	int cid_max;
+	unsigned int cid_min;
+	unsigned int cid_max;
+	unsigned char *cid_bitmap;
 	int netreg_status;
 	struct ofono_netreg *netreg;
 	unsigned int netreg_watch;
@@ -142,6 +143,48 @@ static enum gprs_context_type gprs_context_string_to_type(const char *str)
 	return GPRS_CONTEXT_TYPE_INVALID;
 }
 
+static unsigned int gprs_cid_alloc(struct ofono_gprs *gprs)
+{
+	unsigned int i, j;
+	unsigned int num_cids = gprs->cid_max - gprs->cid_min + 1;
+	unsigned int offset = num_cids / 8;
+	unsigned int max_bit = num_cids % 8;
+
+	for (i = 0; i < offset; i++) {
+		for (j = 0; j < 8; j++) {
+			if (gprs->cid_bitmap[i] & (1 << j))
+				continue;
+
+			gprs->cid_bitmap[i] |= (1 << j);
+			return i * 8 + j + 1;
+		}
+	}
+
+	for (j = 0; j < max_bit; j++) {
+		if (gprs->cid_bitmap[offset] & (1 << j))
+			continue;
+
+		gprs->cid_bitmap[i] |= (1 << j);
+		return offset * 8 + j + 1;
+	}
+
+	return 0;
+}
+
+static void gprs_cid_release(struct ofono_gprs *gprs, unsigned int id)
+{
+	unsigned int offset = (id - 1) / 8;
+	unsigned int bit = (id - 1) % 8;
+
+	if (id > gprs->cid_max)
+		return;
+
+	if (id < gprs->cid_min)
+		return;
+
+	gprs->cid_bitmap[offset] &= ~(1 << bit);
+}
+
 static struct pri_context *gprs_context_by_path(struct ofono_gprs *gprs,
 						const char *ctx_path)
 {
@@ -408,6 +451,10 @@ static void pri_activate_callback(const struct ofono_error *error,
 				telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&gc->pending,
 					__ofono_error_failed(gc->pending));
+
+		gprs_cid_release(ctx->gprs, ctx->context.cid);
+		ctx->context.cid = 0;
+
 		return;
 	}
 
@@ -427,7 +474,6 @@ static void pri_activate_callback(const struct ofono_error *error,
 						DATA_CONTEXT_INTERFACE,
 						"Active", DBUS_TYPE_BOOLEAN,
 						&value);
-
 }
 
 static void pri_deactivate_callback(const struct ofono_error *error, void *data)
@@ -445,6 +491,9 @@ static void pri_deactivate_callback(const struct ofono_error *error, void *data)
 		return;
 	}
 
+	gprs_cid_release(ctx->gprs, ctx->context.cid);
+	ctx->context.cid = 0;
+
 	ctx->active = FALSE;
 	__ofono_dbus_pending_reply(&gc->pending,
 				dbus_message_new_method_return(gc->pending));
@@ -655,10 +704,24 @@ static DBusMessage *pri_set_property(DBusConnection *conn,
 				gc->driver->deactivate_primary == NULL)
 			return __ofono_error_not_implemented(msg);
 
-		gc->pending = dbus_message_ref(msg);
+		if (value) {
+			ctx->context.cid = gprs_cid_alloc(ctx->gprs);
 
-		/* TODO: Find lowest unused CID */
-		ctx->context.cid = 1;
+			if (ctx->context.cid == 0)
+				return __ofono_error_failed(msg);
+
+			if (ctx->context.cid != 1) {
+				ofono_error("Multiple active contexts are"
+						" not yet supported");
+
+				gprs_cid_release(ctx->gprs, ctx->context.cid);
+				ctx->context.cid = 0;
+
+				return __ofono_error_failed(msg);
+			}
+		}
+
+		gc->pending = dbus_message_ref(msg);
 
 		if (value)
 			gc->driver->activate_primary(gc, &ctx->context,
@@ -902,6 +965,30 @@ static void gprs_attached_update(struct ofono_gprs *gprs)
 
 	gprs->attached = attached;
 
+	if (gprs->attached == FALSE) {
+		GSList *l;
+		struct pri_context *ctx;
+
+		for (l = gprs->contexts; l; l = l->next) {
+			ctx = l->data;
+
+			if (ctx->active == FALSE)
+				continue;
+
+			gprs_cid_release(gprs, ctx->context.cid);
+			ctx->context.cid = 0;
+
+			ctx->active = FALSE;
+			pri_reset_context_settings(ctx);
+
+			value = FALSE;
+			ofono_dbus_signal_property_changed(conn, ctx->path,
+						DATA_CONTEXT_INTERFACE,
+						"Active", DBUS_TYPE_BOOLEAN,
+						&value);
+		}
+	}
+
 	path = __ofono_atom_get_path(gprs->atom);
 	value = attached;
 	ofono_dbus_signal_property_changed(conn, path,
@@ -1113,8 +1200,9 @@ static DBusMessage *gprs_set_property(DBusConnection *conn,
 		}
 
 		gprs_netreg_update(gprs);
-	} else
+	} else {
 		return __ofono_error_invalid_args(msg);
+	}
 
 	path = __ofono_atom_get_path(gprs->atom);
 	ofono_dbus_signal_property_changed(conn, path,
@@ -1361,11 +1449,15 @@ void ofono_gprs_status_notify(struct ofono_gprs *gprs,
 	gprs_attached_update(gprs);
 }
 
-void ofono_gprs_set_cid_range(struct ofono_gprs *gprs, int min, int max)
+void ofono_gprs_set_cid_range(struct ofono_gprs *gprs,
+				unsigned int min, unsigned int max)
 {
 	if (gprs == NULL)
 		return;
 
+	g_free(gprs->cid_bitmap);
+	gprs->cid_bitmap = g_new0(unsigned char, (max - min + 8) / 8);
+
 	gprs->cid_min = min;
 	gprs->cid_max = max;
 }
@@ -1406,7 +1498,11 @@ void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
 		if (ctx->context.cid != cid)
 			continue;
 
+		gprs_cid_release(ctx->gprs, ctx->context.cid);
+		ctx->context.cid = 0;
+
 		ctx->active = FALSE;
+		pri_reset_context_settings(ctx);
 
 		value = FALSE;
 		ofono_dbus_signal_property_changed(conn, ctx->path,
@@ -1659,61 +1755,61 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 
 	if ((name = g_key_file_get_string(gprs->settings, group,
 					"Name", NULL)) == NULL)
-		goto err;
+		goto error;
 
 	if ((typestr = g_key_file_get_string(gprs->settings, group,
 					"Type", NULL)) == NULL)
-		goto err;
+		goto error;
 
 	type = gprs_context_string_to_type(typestr);
 	if (type == GPRS_CONTEXT_TYPE_INVALID)
-		goto err;
+		goto error;
 
 	username = g_key_file_get_string(gprs->settings, group,
 						"Username", NULL);
 	if (!username)
-		goto err;
+		goto error;
 
 	if (strlen(username) > OFONO_GPRS_MAX_USERNAME_LENGTH)
-		goto err;
+		goto error;
 
 	password = g_key_file_get_string(gprs->settings, group,
 						"Password", NULL);
 
 	if (!password)
-		goto err;
+		goto error;
 
 	if (strlen(password) > OFONO_GPRS_MAX_PASSWORD_LENGTH)
-		goto err;
+		goto error;
 
 	apn = g_key_file_get_string(gprs->settings, group,
 					"AccessPointName", NULL);
 
 	if (!apn)
-		goto err;
+		goto error;
 
 	if (strlen(apn) > OFONO_GPRS_MAX_APN_LENGTH)
-		goto err;
+		goto error;
 
 	/* Accept empty (just created) APNs, but don't allow other
 	 * invalid ones */
 	if (apn[0] != '\0' && is_valid_apn(apn) == FALSE)
-		goto err;
+		goto error;
 
 	if ((context = pri_context_create(gprs, name, type)) == NULL)
-		goto err;
+		goto error;
 
 	strcpy(context->context.username, username);
 	strcpy(context->context.password, password);
 	strcpy(context->context.apn, apn);
 
 	if (context_dbus_register(context) == FALSE)
-		goto err;
+		goto error;
 
 	gprs->contexts = g_slist_append(gprs->contexts, context);
 	ret = TRUE;
 
-err:
+error:
 	g_free(name);
 	g_free(typestr);
 	g_free(username);
